#include <msp/io/print.h>
#include <msp/strings/utils.h>
#include "output.h"
#include "statement.h"
#include "textwriter.h"

using namespace std;

namespace Msp {
namespace DataFile {

TextWriter::TextWriter(Output &o):
	WriterMode(o)
{
	float_format.showpoint().precision(7);
}

void TextWriter::set_float_precision(unsigned fp)
{
	float_format.precision(fp/4-1);
}

void TextWriter::write(const Statement &st)
{
	write_(st, 0);
}

void TextWriter::write_(const Statement &st, unsigned level)
{
	string indent(level, '\t');

	out.write(format("%s%s", indent, st.keyword));
	for(const Value &v: st.args)
	{
		out.put(' ');
		if(v.get_signature()==StringType::signature)
			out.write(format("\"%s\"", c_escape(v.get<StringType::Store>(), false)));
		else if(v.get_signature()==BoolType::signature)
			out.write(v.get<BoolType::Store>() ? "true" : "false");
		else if(v.get_signature()==IntType::signature)
			out.write(lexical_cast<string>(v.get<IntType::Store>()));
		else if(v.get_signature()==FloatType::signature)
			out.write(lexical_cast<string>(v.get<FloatType::Store>(), float_format));
		else if(v.get_signature()==SymbolType::signature)
		{
			string name = v.get<SymbolType::Store>().name;
			if(isdigit(name[0]))
				out.write("\\"+name);
			else
				out.write(name);
		}
	}
	if(!st.sub.empty())
	{
		out.write(format("\n%s{\n", indent));
		for(const Statement &s: st.sub)
			write_(s, level+1);
		out.write(format("%s}", indent));
	}
	out.write(";\n");
}

} // namespace DataFile
} // namespace Msp
